package com.fzm.admin.controller;

import cn.hutool.core.lang.Assert;
import com.fzm.admin.params.LoginParam;
import com.fzm.common.annotation.Authentication;
import com.fzm.common.constant.RedisConstant;
import com.fzm.common.constant.TokenConstant;
import com.fzm.common.entity.Admin;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.interceptor.LoginUserInfo;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.AdminService;
import com.fzm.common.utils.JwtUtil;
import com.fzm.common.utils.RedisUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author tangtuo
 * @date 2021/7/5 9:48
 */
@RestController
@RequestMapping("/admin")
@Api(tags = "后台用户管理")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AdminController {

    private final AdminService adminService;
    private final RedisUtil redisUtil;
    private final PasswordEncoder passwordEncoder;

    @PostMapping("/login")
    @ApiOperation(value = "登录")
    public ResponseModel<Map<String, Object>> login(@Validated @RequestBody LoginParam loginParam) {
        Admin admin = adminService.login(loginParam.getUsername(), loginParam.getPassword());
        Integer freezeState = admin.getFreezeState();
        if (freezeState != null && freezeState == 1) {
            throw GlobalException.newException(ResultCode.FREEZE);
        }

        String token = JwtUtil.generateToken(admin);
        redisUtil.setEx(RedisConstant.ADMIN_USER_TOKEN_PREFIX + admin.getId(), token, 24L, TimeUnit.HOURS);
        HashMap<String, Object> result = new HashMap<>();
        result.put("user", admin);
        result.put(TokenConstant.TOKEN_HEADER, token);

        Integer needResetPassword = admin.getNeedResetPassword();
        if (needResetPassword != null && needResetPassword == 1) {
            return ResponseModel.fail(ResultCode.RESET_PASSWORD, result);
        }
        return ResponseModel.success(result);
    }

    @Authentication
    @PostMapping("/password")
    @ApiOperation(value = "修改密码")
    public ResponseModel<Object> password(@RequestBody LoginParam loginParam) {
        String password = loginParam.getPassword();
        Assert.notBlank(password, "密码不能为空");

        String username = LoginUserInfo.getLoginUser().getUsername();

        Admin admin = new Admin();
        admin.setUsername(username);
        admin.setPassword(passwordEncoder.encode(password));
        admin.setNeedResetPassword(0);

        adminService.updateByUserNameIf(admin);

        return ResponseModel.success();
    }
}
