package com.fzm.common.open.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.entity.Authority;
import com.fzm.common.entity.OpusCategory;
import com.fzm.common.entity.dto.CopyrightRequest;
import com.fzm.common.entity.dto.CopyrightResponse;
import com.fzm.common.enums.CopyrightApplyState;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.interceptor.LoginUserInfo;
import com.fzm.common.mapper.CopyrightApiMapper;
import com.fzm.common.open.entity.dataobject.*;
import com.fzm.common.open.entity.openobject.*;
import com.fzm.common.open.entity.queryobject.CopyrightApiMapperQO;
import com.fzm.common.open.entity.queryobject.CopyrightApiQO;
import com.fzm.common.open.entity.queryobject.CopyrightStateQO;
import com.fzm.common.open.entity.viewobject.CopyrightApiVO;
import com.fzm.common.open.service.*;
import com.fzm.common.properties.CopyrightProperties;
import com.fzm.common.service.AuthorityService;
import com.fzm.common.service.OpusCategoryService;
import com.fzm.common.utils.CopyrightSignUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.Redisson;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TreeMap;
import java.util.concurrent.TimeUnit;

/**
 * @author tangtuo
 * @date 2021/12/9 11:40
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class CopyrightApiServiceImpl extends ServiceImpl<CopyrightApiMapper, CopyrightApiDO> implements CopyrightApiService {
    private final CopyrightApiFileService copyrightApiFileService;
    private final CopyrightApiOwnerRelationService copyrightApiOwnerRelationService;
    private final CopyrightApiAuthorService copyrightApiAuthorService;
    private final CopyrightApiAuthorityRelationService copyrightApiAuthorityRelationService;

    private final OpusCategoryService opusCategoryService;
    private final AuthorityService authorityService;

    private final CopyrightApiMapper copyrightApiMapper;

    private final CopyrightProperties copyrightProperties;

    @Resource
    private Redisson redisson;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer apply(BaseOO<CopyrightOO> baseOO, OpenKeyDO openKeyDO) {
        String appKey = baseOO.getAppKey();
        String appName = openKeyDO.getAppName();
        String bizId = baseOO.getBizId();
        CopyrightOO copyrightOO = baseOO.getData();

        CopyrightApiDO copyrightApiDO = BeanUtil.copyProperties(copyrightOO, CopyrightApiDO.class);
        copyrightApiDO.setRegisterState(CopyrightApplyState.TO_BE_REVIEWED.getCode());
        copyrightApiDO.setAppKey(appKey);
        copyrightApiDO.setAppName(appName);
        copyrightApiDO.setBizId(bizId);
        copyrightApiDO.setApplyTime(new Date());
        save(copyrightApiDO);

        // 保存附件
        List<CopyrightFileOO> fileOOS = copyrightOO.getFiles();
        ArrayList<CopyrightApiFileDO> files = new ArrayList<>(fileOOS.size());
        for (CopyrightFileOO fileOO : fileOOS) {
            CopyrightApiFileDO fileDO = BeanUtil.copyProperties(fileOO, CopyrightApiFileDO.class);
            fileDO.setCopyrightId(copyrightApiDO.getId());
            files.add(fileDO);
        }
        copyrightApiFileService.saveBatch(files);

        // 保存著作权人信息
        List<CopyrightOwnerOO> ownerOOS = copyrightOO.getOwners();
        ArrayList<CopyrightApiOwnerRelationDO> owners = new ArrayList<>(ownerOOS.size());
        for (CopyrightOwnerOO ownerOO : ownerOOS) {
            CopyrightApiOwnerRelationDO ownerRelationDO = BeanUtil.copyProperties(ownerOO, CopyrightApiOwnerRelationDO.class);
            ownerRelationDO.setCopyrightId(copyrightApiDO.getId());
            owners.add(ownerRelationDO);
        }
        copyrightApiOwnerRelationService.saveBatch(owners);

        // 保存作者信息
        List<CopyrightAuthorOO> authorOOS = copyrightOO.getAuthors();
        ArrayList<CopyrightApiAuthorDO> authors = new ArrayList<>(authorOOS.size());
        for (CopyrightAuthorOO authorOO : authorOOS) {
            CopyrightApiAuthorDO authorDO = BeanUtil.copyProperties(authorOO, CopyrightApiAuthorDO.class);
            authorDO.setCopyrightId(copyrightApiDO.getId());
            authors.add(authorDO);
        }
        copyrightApiAuthorService.saveBatch(authors);

        // 保存权力信息
        List<Integer> authorityIds = copyrightOO.getAuthorityIds();
        ArrayList<CopyrightApiAuthorityRelationDO> relations = new ArrayList<>(authorityIds.size());
        for (Integer authorityId : authorityIds) {
            CopyrightApiAuthorityRelationDO relationDO = new CopyrightApiAuthorityRelationDO();
            relationDO.setCopyrightId(copyrightApiDO.getId());
            relationDO.setAuthorityId(authorityId);
            relations.add(relationDO);
        }
        copyrightApiAuthorityRelationService.saveBatch(relations);

        return copyrightApiDO.getId();
    }

    @Override
    public CopyrightApiDO getByAppKeyAndBizId(String appKey, String bizId) {
        QueryWrapper<CopyrightApiDO> wrapper = new QueryWrapper<>();
        wrapper.eq("app_key", appKey);
        wrapper.eq("biz_id", bizId);
        return getOne(wrapper);
    }

    @Override
    public List<CopyrightApiDO> getByAppKeyAndBizIds(CopyrightStateQO stateQO) {
        return copyrightApiMapper.getByAppKeyAndBizIds(stateQO);
    }

    @Override
    public PageInfo<CopyrightApiDO> pages(CopyrightApiQO copyrightApiQO) {
        CopyrightApiMapperQO mapperQO = BeanUtil.copyProperties(copyrightApiQO, CopyrightApiMapperQO.class);

        String startDateStr = copyrightApiQO.getStartDate();
        if (StringUtils.isNotBlank(startDateStr)) {
            LocalDate startDate = LocalDate.parse(startDateStr);
            LocalDateTime startDateTime = startDate.atStartOfDay();
            mapperQO.setStartDate(startDateTime);
        }
        String endDateStr = copyrightApiQO.getEndDate();
        if (StringUtils.isNotBlank(endDateStr)) {
            LocalDate endDate = LocalDate.parse(endDateStr);
            LocalDateTime endDateTime = endDate.atTime(23, 59, 59);
            mapperQO.setEndDate(endDateTime);
        }

        PageHelper.startPage(copyrightApiQO.getPage(), copyrightApiQO.getSize());

        List<CopyrightApiDO> list = copyrightApiMapper.getList(mapperQO);
        return new PageInfo<>(list);
    }


    @Override
    public CopyrightApiVO getDetail(Integer id) {
        CopyrightApiDO apiDO = getById(id);
        if (apiDO == null) {
            return null;
        }
        CopyrightApiVO apiVO = new CopyrightApiVO();
        apiVO.setCopyright(apiDO);

        Integer opusCategoryId = apiDO.getOpusCategoryId();
        OpusCategory category = opusCategoryService.getById(opusCategoryId);
        apiVO.setOpusCategory(category);

        List<CopyrightApiFileDO> fileDOS = copyrightApiFileService.getByCopyrightId(id);
        apiVO.setFiles(fileDOS);

        List<CopyrightApiAuthorDO> authorDOS = copyrightApiAuthorService.getByCopyrightId(id);
        apiVO.setAuthors(authorDOS);

        List<CopyrightApiOwnerRelationDO> relationDOS = copyrightApiOwnerRelationService.getByCopyrightId(id);
        apiVO.setOwners(relationDOS);

        List<Authority> authorities1 = authorityService.getApiByCopyrightId(id);
        apiVO.setAuthorities(authorities1);

        return apiVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void pass(Integer id) {
        RLock lock = redisson.getLock("copyright:api:apply:" + id);
        boolean tryLock;
        try {
            tryLock = lock.tryLock(10, TimeUnit.SECONDS);
        } catch (InterruptedException e) {
            throw GlobalException.newException(ResultCode.FAILED, "操作频繁");
        }
        if (!tryLock) {
            throw GlobalException.newException(ResultCode.FAILED, "操作频繁");
        }
        try {
            // 请求版权局接口
            CopyrightApiVO copyrightApiVO = this.getDetail(id);

            CopyrightRequest copyrightRequest = new CopyrightRequest(copyrightApiVO, copyrightProperties.getAppId());
            TreeMap<String, String> param = this.beanToMap(copyrightRequest, copyrightProperties.getAppSecret());
            String sign = CopyrightSignUtil.generateSign(param);
            copyrightRequest.setSign(sign);
            String json = JSONUtil.toJsonStr(copyrightRequest);
            log.info("请求版权局接口, 接口地址:{}, 请求参数:{}", copyrightProperties.getApplyUrl(), json);
            String response = HttpUtil.post(copyrightProperties.getApplyUrl(), json, 10000);
            log.info("版权局响应内容:{}", response);
            if (StringUtils.isBlank(response)) {
                throw GlobalException.newException(ResultCode.COPYRIGHT_FAILED, "版权局接口无响应,请稍后重试");
            }
            CopyrightResponse copyrightResponse = JSONUtil.toBean(response, CopyrightResponse.class);
            if (copyrightResponse == null) {
                throw GlobalException.newException(ResultCode.COPYRIGHT_FAILED, "版权局接口无响应,请稍后重试");
            }
            if (!copyrightResponse.isSuccess()) {
                throw GlobalException.newException(ResultCode.COPYRIGHT_FAILED, copyrightResponse.getMessage());
            }
            // 申请成功
            String serialCode = copyrightResponse.getSerial_code();

            CopyrightApiDO apiDO = new CopyrightApiDO();
            apiDO.setId(id);
            apiDO.setSerialNum(serialCode);
            apiDO.setRegisterState(CopyrightApplyState.SUBMITTED.getCode());
            apiDO.setRejectReason("");
            apiDO.setPassTime(new Date());
            apiDO.setCheckUsername(LoginUserInfo.getLoginUser().getUsername());
            updateById(apiDO);
        } finally {
            lock.unlock();
        }

    }


    @Override
    public void reject(Integer id, String rejectReason) {
        CopyrightApiDO apiDO = new CopyrightApiDO();
        apiDO.setId(id);
        apiDO.setRegisterState(CopyrightApplyState.REJECTED.getCode());
        apiDO.setRejectReason(rejectReason);
        apiDO.setCheckUsername(LoginUserInfo.getLoginUser().getUsername());
        updateById(apiDO);
    }

    @Override
    public List<String> getSerialCodes() {
        return copyrightApiMapper.getSerialCodes();
    }

    @Override
    public CopyrightApiDO getBySerialNum(String serialCode) {
        QueryWrapper<CopyrightApiDO> wrapper = new QueryWrapper<>();
        wrapper.eq("serial_num", serialCode);
        return getOne(wrapper);
    }

    private TreeMap<String, String> beanToMap(CopyrightRequest copyrightRequest, String app_secret) {
        TreeMap<String, String> treeMap = JSONUtil.toBean(JSONUtil.toJsonStr(copyrightRequest), new TypeReference<TreeMap<String, String>>() {
        }, true);
        treeMap.put("app_secret", app_secret);
        return treeMap;
    }

}
