package com.fzm.common.utils;

import cn.hutool.core.lang.TypeReference;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.json.JSONUtil;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * @author wt
 * @date 2022/4/24
 */
public class OpenSignUtils {

    public static void verify(String appKey, String appKeySecret, String bizId, Object obj, String signature) {
        TreeMap<String, Object> param = new TreeMap<>();
        param.put("appKey", appKey);
        param.put("appKeySecret", appKeySecret);
        param.put("bizId", bizId);

        String mapToStr = mapToStr(toTreeMap(obj));
        String bizHash = SecureUtil.md5(mapToStr);
        param.put("bizHash", bizHash);

        mapToStr = mapToStr(param);

        String md5 = SecureUtil.md5(mapToStr);
        boolean equals = signature.equals(md5);
        if (!equals) {
            throw GlobalException.newException(ResultCode.signature_wrong);
        }
    }

    public static TreeMap<String, Object> toTreeMap(Object obj) {
        return JSONUtil.toBean(JSONUtil.toJsonStr(obj), new TypeReference<TreeMap<String, Object>>() {
        }, true);
    }

    private static String mapToStr(TreeMap<String, Object> treeMap) {
        StringBuilder sb = new StringBuilder();

        for (Map.Entry<String, Object> entry : treeMap.entrySet()) {
            String k = entry.getKey();
            Object v = entry.getValue();

            if (v instanceof String) {
                sb.append(k).append("=").append(v).append("&");
                continue;
            }

            if (k.endsWith("Date")) {
                long v1 = Long.parseLong(v.toString());
                String s = "";
                if (v1 > 0) {
                    s = dateFormat(timestamp2DateTime(v1));
                }
                sb.append(k).append("=").append(s).append("&");
                continue;
            }

            if (v instanceof Integer) {
                sb.append(k).append("=").append(v).append("&");
                continue;
            }

            if (v instanceof Long) {
                sb.append(k).append("=").append(v).append("&");
                continue;
            }
            if (v == null) {
                sb.append(k).append("=").append("&");
                continue;
            }

            if (v instanceof List) {
                List list = (List) v;
                Object o1 = list.get(0);
                if (o1 instanceof Integer) {
                    sb.append(k).append("=").append(list).append("&");
                    continue;
                }

                StringBuilder sbSub = new StringBuilder();
                for (Object o : list) {
                    TreeMap<String, Object> map = toTreeMap(o);
                    sbSub.append(mapToStr(map)).append("&");
                }
                String sbSubStr = sbSub.deleteCharAt(sbSub.length() - 1).toString();
                String md5 = SecureUtil.md5(sbSubStr);
                sb.append(k).append("=").append(md5).append("&");
            }
        }
        return sb.deleteCharAt(sb.length() - 1).toString();
    }

    private static final DateTimeFormatter ymd = DateTimeFormatter.ofPattern("yyyy-MM-dd");

    private static LocalDateTime timestamp2DateTime(long timestamp) {
        Instant instant = Instant.ofEpochMilli(timestamp);
        return LocalDateTime.ofInstant(instant, ZoneId.of("Asia/Shanghai"));
    }

    public static String dateFormat(LocalDateTime dateTime) {
        return dateTime.format(ymd);
    }
}
